function [K, U] = fisher_kernel_hmm(X, model)
%FISHER_KERNEL_HMM Compute Fisher kernel for Hidden Markov Model
%
%   [K, U] = fisher_kernel_hmm(X, model)
%
% Compute Fisher kernel for dataset X under the Hidden Markov Model model.
% The function also returns the gradient representation in U.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego


    % Compute Fisher score (per instance)
    log_domain = true;
    N = length(X);
    x = [model.pi(:); model.A(:); model.mu(:)];
    if any(strcmpi(model.emission, {'gaussian', 'gaussian_mix'}))
        x = [x; model.sigma(:)];
    end
    if strcmpi(model.emission, 'gaussian_mix')
        x = [x; model.mix(:)];
    end
    U = zeros(numel(model.pi) + numel(model.A) + numel(model.mu), N);
    for i=1:N
        [~, U(:,i)] = hmm_grad(x, X{i}, model, log_domain, true, true, true, false, false);
    end
    
    % Normalize L2-norm of gradients
    U = bsxfun(@rdivide, U, max(sqrt(sum(U .^ 2, 1)), realmin));
    
    % Compute approximate Fisher kernel
    K = U' * U;
    
%     % Compute corresponding posterior probabilities
%     post = sum(bsxfun(@times, pi, bernoulli_density(X, A)), 2);
%     post = bsxfun(@rdivide, post, sum(post, 2));
%     
%     % Compute Fisher information matrix
%     I = U * bsxfun(@times, post, U');
%     
%     % Compute Fisher kernel
%     K = U' * (I \ U);                       % = U' * inv(I) * U;

    % Transpose U is required
    if nargout > 1
        U = U';
    end
    