function [C, dC] = gauss_grad(x, X, D)


    % Decode solution
    mu    = x(1:D);
    sigma = reshape(x(1+D:end), [D D]);
    N = size(X, 2);

    % Compute log-likelihood of data
    Xi = bsxfun(@minus, X, mu);
    const = (-.5 * N * D * log(2 * pi)) - (N / 2) * log(det(sigma));
    L = const - .5 * sum(sum((sigma \ Xi) .* Xi, 1));
    C = -L;
    
    % Only compute gradients if required
    if nargout > 1
        
        % Compute gradient
        dMu = sum(sigma \ Xi, 2);
        dSigma = -(N / 2) * inv(sigma) + (N / 2) * inv(sigma) * (Xi / N * Xi') * inv(sigma);
        
        % Encode solution
        dC = -[dMu(:); dSigma(:)];
    end
    