function p = gaussian_emission(X, model)
%GAUSSIAN_EMISSION Compute Gaussian emission log-probabilities 
%
%   p = gaussian_emission(X, model)
%
% Compute Gaussian emission log-probabilities for time series X under the
% specified Hidden Markov Model.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego


    % Initialize some variables
    assert(strcmpi(model.emission, 'gaussian'));
    [D, N] = size(X);
    K = numel(model.pi);
    p = zeros(K, N);  
    
    % Compute emission log-probability for each Gaussian (state) and time step
    for k=1:K           
        const = -(D ./ 2) .* log(2 * pi) - log(max(sqrt(max(0, det(model.sigma(:,:,k)))), realmin));
        Xi = bsxfun(@minus, X, model.mu(:,k));
        p(k,:) = const - .5 .* sum((model.sigma(:,:,k) \ Xi) .* Xi, 1);
    end
    p = real(p);
    