function p = multinomial_emission(X, model)
%MULTINOMIAL_EMISSION Compute multinomial emission log-probabilities 
%
%   p = multinomial_emission(X, model)
%
% Compute multinomial emission log-probabilities for time series X under the
% specified Hidden Markov Model.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego

    
    % Initialize some variables
    assert(strcmpi(model.emission, 'multinomial'));
    K = size(model.A, 1);
    N = size(X, 2);
    p = zeros(K, N);
    
    % Compute emission probability for each multinomial (= state)
    for k=1:K
        p(k,:) = sum(bsxfun(@times, log(max(model.mu(:,k), eps)), X), 1);
    end        
