function err = nn_error(X, labels, M)
%NN_ERROR Compute nearest neighbor error
%
%   err = nn_error(X, labels, metric)
%
% Computes the nearest neighbor error for dataset X with labels using a
% Mahalanobis metric. If metric is not defined, it is assumed to be the
% identity matrix
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego


    % Compute pairwise distances
    if ~exist('M', 'var') || isempty(M)
        sum_X = sum(X .^ 2, 2);
        D = bsxfun(@plus, sum_X, bsxfun(@plus, sum_X', -2 * (X * X')));
    else
        N = size(X, 1);
        D = zeros(N, N);
        for n=1:N
            diff = bsxfun(@minus, X(n,:), X);
            if any(size(M) == 1)
                D(:,n) = sum(bsxfun(@times, diff, M') .* diff, 2);
            else
                D(:,n) = sum((diff * M) .* diff, 2);
            end
        end 
    end
    
    % Retrieve nearest neighbor indices
    [foo, ind] = sort(D, 2, 'ascend');
    ind = ind(:,2);
    
    % Compute nearest neighbor error
    err = sum(labels(ind) ~= labels) ./ numel(labels);
    