function [E, E_bias] = train_logreg(train_X, train_labels, lambda)
%TRAIN_LOGREG Trains an L2-regularized logistic regressor
%
%   [E, E_bias] = train_logreg(train_X, train_labels, lambda)
%
% Trains an L2-regularized logistic regressor.


    addpath(genpath('minFunc'));
    

    % Initialize solution
    D = size(train_X, 1);
    [lablist, foo, train_labels] = unique(train_labels);
    K = length(lablist);
    E = randn(D, K) * .0001;
    E_bias = zeros(1, K);
    
    % Compute positive part of gradient
    pos_E = zeros(D, K);
    pos_E_bias = zeros(1, K);
    for k=1:K
        pos_E(:,k) = sum(train_X(:,train_labels == k), 2);
        pos_E_bias(k) = sum(train_labels == k);
    end
    
%     % Check gradient
%     checkgrad('logreg_grad', [E(:); E_bias(:)], 1e-7, train_X, train_labels, lambda, pos_E, pos_E_bias)
    
    % Perform learning using L-BFGS
    options.Method = 'lbfgs';
    options.Display = 'on';
    options.TolFun = 1e-5;
    options.TolX = 1e-5;
    options.MaxIter = 5000;   
    options.Display = 'final';
    x = minFunc(@logreg_grad, [E(:); E_bias(:)], options, train_X, train_labels, lambda, pos_E, pos_E_bias);
    
    % Decode solution
    E = reshape(x(1:D * K), [D K]);
    E_bias = reshape(x(D * K + 1:end), [1 K]);
