function model = update_gaussian(X, model, gamma)
%UPDATE_GAUSSIAN Given responsibilities, update Gaussian emissions distr.
%
%   model = update_gaussian(X, model, gamma)
%
% Given a time series X and the corresponding responsibilities gamma, the
% function updates the parameters of the Gaussian emission distribution of
% the Hidden Markov Model specified in model.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego

    
    % Initialize some variables
    assert(strcmpi(model.emission, 'gaussian'));
    assert(iscell(X) && iscell(gamma) && length(X) == length(gamma))
    D = size(X{1}, 1);
    K = length(model.pi);
    sum2 = zeros(K, 1);
    min_std = 1e-5;
    
    % Update means of the Gaussians    
    for k=1:K
        sum1 = zeros(D, 1);
        for i=1:length(X)
            sum1 = sum1 + sum(bsxfun(@times, gamma{i}(k,:), X{i}), 2);
            sum2(k) = sum2(k) + sum(gamma{i}(k,:));
        end
        model.mu(:,k) = sum1 ./ max(sum2(k), eps);
    end
    
    % Update covariances of the Gaussians
    for k=1:K
        sum1 = zeros(D, D);
        for i=1:length(X)
            Xk = bsxfun(@minus, X{i}, model.mu(:,k));
            sum1 = sum1 + bsxfun(@times, gamma{i}(k,:), Xk) * Xk';
        end
        model.sigma(:,:,k) = real(sum1 ./ max(sum2(k), eps)) + min_std * eye(D);
    end
    
    % Make sure parameters are real
    model.mu = real(model.mu);
    model.sigma = real(model.sigma);
    