function model = update_gaussian_mix(X, model, gamma)
%UPDATE_GAUSSIAN_MIX Given responsibilities, update Gaussian mixture emissions
%
%   model = update_gaussian_mix(X, model, gamma)
%
% Given a time series X and the corresponding responsibilities gamma, the
% function updates the parameters of the Gaussian mixture emission 
% distribution of the Hidden Markov Model specified in model.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego

    
    warning('This function is not yet updated to deal with multiple time series.');
    

    % Initialize some variables
    assert(strcmpi(model.emission, 'gaussian_mix'));
    assert(iscell(X) && iscell(gamma) && length(X) == length(gamma))
    [D, N] = size(X{1});
    K = length(model.pi);
    L = size(model.mix, 1);
    post = zeros(N, L, K);
    
    % Compute the posterior within each mixture
    for k=1:K
        for l=1:L
            const = 1 ./ ((2 * pi) .^ (D ./ 2) .* sqrt(det(model.sigma(:,:,l,k))));
            Xi = bsxfun(@minus, X, model.mu(:,l,k));
            post(:,l,k) =  model.mix(l, k) .* const .* exp(-.5 * sum((model.sigma(:,:,l,k) \ Xi) .* Xi, 1));
        end
    end
    post = bsxfun(@rdivide, post, sum(post, 3));

    % Update means of the Gaussians
    for k=1:K
        for l=1:L
            model.mu(:,l,k) = sum(bsxfun(@times, gamma(k,:) .* post(:,l,k)', X), 2) ./ sum(gamma(k,:) + realmin);
        end
    end

    % Update covariances of the Gaussians
    reg = eye(D) .* 1e-7;
    for k=1:K
        for l=1:L
            Xk = bsxfun(@minus, X, model.mu(:,l,k));
            model.sigma(:,:,l,k) = reg + (bsxfun(@times, gamma(k,:) .* post(:,l,k)', Xk) * Xk') ./ sum(gamma(k,:) + realmin);
        end
    end
    
    % Update the mixing weights of the mixtures
    for k=1:K
        model.mix(:,k) = sum(bsxfun(@times, gamma(k,:)', post(:,:,k)), 1)' ./ ...
                     sum(sum(bsxfun(@times, gamma(k,:)', post(:,:,k))));
    end
    
    % Make sure parameters are real
    model.mix = real(model.mix);
    model.mu = real(model.mu);
    model.sigma = real(model.sigma);
    