function model = update_multinomial(X, model, gamma)
%UPDATE_MULTINOMIAL Given responsibilities, update multinomial emissions
%
%   model = update_multinomial(X, model, gamma)
%
% Given a time series X and the corresponding responsibilities gamma, the
% function updates the parameters of the multinomial emission distribution 
% of the Hidden Markov Model specified in model.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego

    
    % Initialize some variables
    assert(strcmpi(model.emission, 'multinomial'));
    assert(iscell(X) && iscell(gamma) && length(X) == length(gamma))
    [D, K] = size(model.mu);

    % Update the multinomal parameters
    for k=1:K
        sum1 = zeros(D, 1);
        sum2 = 0;
        for i=1:length(X)
             sum1 = sum1 + sum(bsxfun(@times, gamma{i}(k,:), X{i}), 2);
             sum2 = sum2 + sum(gamma{i}(k,:));
        end
        model.mu(:,k) = sum1 ./ sum2;
    end