function [sequence, L] = viterbi(X, model)
%VITERBI Performs Viterbi algorithm to find most likely state sequence
%
%   [sequence, L] = viterbi(X, model)
%
% Performs the Viterbi algorithm on time series X in the Hidden Markov
% Model specified in model, to find the most likely underlying state 
% sequence. The log-likelihood of the sequence is returned in L.
%
%
% (C) Laurens van der Maaten, 2010
% University of California, San Diego


    % Initialize some variables
    N = size(X, 2);
    K = numel(model.pi);
    omega = zeros(K, N);
    ind = zeros(K, N);
    sequence = zeros(1, N);
    if N == 0
        L = [];
        return;
    end
    
    % Compute emission probabilities
    eval(['emission = ' model.emission '_emission(X, model);']);
    
    % Compute message for first hidden variable
    omega(:,1) = log(model.pi + realmin) + emission(:,1);
    
    % Perform forward pass
    log_A = log(model.A + eps);
    for n=2:N
        [omega(:,n), ind(:,n)] = max(bsxfun(@plus, log_A, omega(:,n - 1)), [], 1);
        omega(:,n) = omega(:,n) + emission(:,n);
    end
    
    % Perform backtracking to determine final sequence
    [L, sequence(N)] = max(real(omega(:,N)));
    for n=N - 1:-1:1
        sequence(n) = ind(sequence(n + 1), n + 1);
    end
